import unittest

from emailage import validation


class ValidationTest(unittest.TestCase):
    
    def setUp(self):
        self.correct_email = 'test+emailage@example.com'
        self.incorrect_email = 'test+example.com'
        self.correct_ip = '1.234.56.7'
        self.incorrect_ip = '1.23.456.7890'

        self.ipv6_samples_valid = [
            '2001:db8:a0b:12f0::1',
            'FE80:0000:0000:0000:0202:B3FF:FE1E:8329',
            '2607:f0d0:1002:0051:0000:0000:0000:0004',
            'FE80::0202:B3FF:FE1E:8329',
            '2001:0:9d38:90d7:301f:1c10:3f57:fe64'
        ]

        self.ipv6_samples_invalid = [
            '1200::AB00:1234::2552:7777:1313',
            '20::afd34:32::42',
            '1200:0000:AB00:1234:O000:2552:7777:1313'
        ]

        self.ipv4_samples_valid = [
            '255.255.255.255',
            '174.70.13.43',
            '208.67.222.222',
            '192.168.1.155',
            '10.232.222.212',
            '10.123.56.25',
            '155.1.0.1',
            '10.123.0.25',
            '255.0.1.1',
            '255.1.0.255',
            '255.255.255.0',
        ]

        self.ipv4_samples_invalid = [
            '1.1.1:2823',
            '10001.fff45.222.1000:01',
            '192.168.1.1:443',
            '0.0.0.1',
            '0.0.0.0',
            '256.0.0.0',
            '256.0.0.1',
            '155.0.0.',
            '256.0.0.',
            '256.255.255.255',
            '25.255.255.256',
            '255.256.1.1',
            '255.255...',
            '255.255.',
            '255.255.255',
            '255.255.255.',

            '255.1.255.260'
        ]

    def test_validates_valid_ipv4(self):
        failed_ips = []
        ip_data_source = self.ipv4_samples_valid
        for ipv4 in ip_data_source:
            try:
                validation.assert_ip(ipv4)
            except ValueError as exc:
                failed_ips.append(ipv4)

        self.assertListEqual([], failed_ips)

    def test_validates_invalid_ipv4(self):
        failed_ips = []
        ip_data_source = self.ipv4_samples_invalid
        for ipv4 in ip_data_source:
            try:
                self.assertRaises(ValueError, validation.assert_ip, ipv4)
            except Exception as exc:
                failed_ips.append(ipv4)

        self.assertListEqual([], failed_ips)

    def test_validates_email(self):
        validation.assert_email(self.correct_email)
        self.assertRaises(ValueError, validation.assert_email, self.incorrect_email)
        self.assertRaises(ValueError, validation.assert_email, self.correct_ip)
        self.assertRaises(ValueError, validation.assert_email, self.incorrect_ip)

    def test_validates_ip(self):
        for entry in self.ipv6_samples_valid:
            validation.assert_ip(entry)

        self.assertRaises(ValueError, validation.assert_ip, self.correct_email)
        self.assertRaises(ValueError, validation.assert_ip, self.incorrect_email)


if __name__ == '__main__':
    unittest.main()



"""OAuth1 module written according to http://oauth.net/core/1.0/#signing_process"""
import base64
import hmac
import requests  # requests must be loaded so that urllib receives the parse module
import time
import urllib

from hashlib import sha1
from six import b
from uuid import uuid4

use_parse_quote = not hasattr(urllib, 'quote')

if use_parse_quote:
    _quote_func = urllib.parse.quote
else:
    _quote_func = urllib.quote


def _quote(obj):
    return _quote_func(str(obj), safe='')


def normalize_query_parameters(params):
    """9.1.1.  Normalize Request Parameters"""
    return '&'.join(map(lambda pair: '='.join([_quote(pair[0]), _quote(pair[1])]), sorted(params.items())))


def concatenate_request_elements(method, url, query):
    """9.1.3.  Concatenate Request Elements"""
    return '&'.join(map(_quote, [str(method).upper(), url, query]))


def hmac_sha1(base_string, hmac_key):
    """9.2.  HMAC-SHA1"""
    hash = hmac.new(b(hmac_key), b(base_string), sha1)
    return hash.digest()


def encode(digest):
    """9.2.1.  Generating Signature"""
    return base64.b64encode(digest).decode('ascii').rstrip('\n')


def add_oauth_entries_to_fields_dict(secret, params, nonce=None, timestamp=None):
    """ Adds dict entries to the user's params dict which are required for OAuth1.0 signature generation

        :param secret: API secret
        :param params: dictionary of values which will be sent in the query
        :param nonce: (Optional) random string used in signature creation, uuid4() is used if not provided
        :param timestamp: (Optional) integer-format timestamp, time.time() is used if not provided
        :return: dict containing params and the OAuth1.0 fields required before executing signature.create

        :type secret: str
        :type params: dict
        :type nonce: str
        :type timestamp: int

        :Example:

        >>> from emailage.signature import add_oauth_entries_to_fields_dict
        >>> query_params = dict(user_email='registered.account.user@yourcompany.com',\
            query='email.you.are.interested.in@gmail.com'\
        )
        >>> query_params = add_oauth_entries_to_fields_dict('YOUR_API_SECRET', query_params)
        >>> query_params['oauth_consumer_key']
        'YOUR_API_SECRET'
        >>> query_params['oauth_signature_method']
        'HMAC-SHA1'
        >>> query_params['oauth_version']
        1.0
    """
    if nonce is None:
        nonce = uuid4()
    if timestamp is None:
        timestamp = int(time.time())

    params['oauth_consumer_key'] = secret
    params['oauth_nonce'] = nonce
    params['oauth_signature_method'] = 'HMAC-SHA1'
    params['oauth_timestamp'] = timestamp
    params['oauth_version'] = 1.0

    return params


def create(method, url, params, hmac_key):
    """ Generates the OAuth1.0 signature used as the value for the query string parameter 'oauth_signature'
    
        :param method: HTTP method that will be used to send the request ( 'GET' | 'POST' ); EmailageClient uses GET
        :param url: API domain and endpoint up to the ?
        :param params: user-provided query string parameters and the OAuth1.0 parameters
            :method add_oauth_entries_to_fields_dict:
        :param hmac_key: for Emailage users, this is your consumer token with an '&' (ampersand) appended to the end

        :return: str value used for oauth_signature

        :type method: str
        :type url: str
        :type params: dict
        :type hmac_key: str

        :Example:

        >>> from emailage.signature import add_oauth_entries_to_fields_dict, create
        >>> your_api_key = 'SOME_KEY'
        >>> your_hmac_key = 'SOME_SECRET' + '&'
        >>> api_url = 'https://sandbox.emailage.com/emailagevalidator/'
        >>> query_params = { 'query': 'user.you.are.validating@gmail.com', 'user_email': 'admin@yourcompany.com' }
        >>> query_params = add_oauth_entries_to_fields_dict(your_api_key, query_params)
        >>> query_params['oauth_signature'] = create('GET', api_url, query_params, your_hmac_key)

    """

    query = normalize_query_parameters(params)
    base_string = concatenate_request_elements(method, url, query)
    digest = hmac_sha1(base_string, hmac_key)
    return encode(digest)



"""Samples given according to http://oauth.net/core/1.0/#sig_base_example"""
from __future__ import print_function
import unittest

from emailage import signature


class SignatureTest(unittest.TestCase):
    
    def setUp(self):
        self.method = 'GET'
        self.url = 'http://photos.example.net/photos'
        self.params = dict(
            oauth_consumer_key='dpf43f3p2l4k3l03',
            oauth_token='nnch734d00sl2jdk',
            oauth_signature_method='HMAC-SHA1',
            oauth_timestamp=1191242096,
            oauth_nonce='kllo9940pd9333jh',
            oauth_version=1.0,
            file='vacation.jpg',
            size='original'
        )
        self.hmac_key = 'kd94hf93k423kf44&pfkkdhi9sl3r4s00'

        self.test_query_email = 'tumbled.email+inbox@gmail.com'
        self.test_query_ip = '13.25.10.245'

        self.no_spaces_params = {
            'first_name': 'Johann',
            'last_name': 'Vandergrift',
            'phone': '+14805559163'
        }

        self.spaces_params_first_name = {
            'first_name': 'Johann Paulus',
            'last_name': 'Vandergrift',
            'phone': '+14805559163'
        }

        self.spaces_params_last_name = {
            'first_name': 'Johann',
            'last_name': 'van der Grift',
            'phone': '+14805559163'
        }

        self.responseStatusSuccess = {
            'status': 'success',
            'errorCode': '0',
            'description': ''
        }

    def _add_test_oauth_params_to_request_dict(self, request_dict):
        request_dict['oauth_consumer_key'] = 'dpf43f3p2l4k3l03'
        request_dict['oauth_token'] = 'nnch734d00sl2jdk'
        request_dict['oauth_signature_method'] = 'HMAC-SHA1'
        request_dict['oauth_timestamp'] = 1191242096
        request_dict['oauth_nonce'] = 'kllo9940pd9333jh'
        request_dict['oauth_version'] = 1.0
        return request_dict

    def test_normalizes_query_parameters(self):
        query = signature.normalize_query_parameters(self.params)
        self.assertEqual(query, 'file=vacation.jpg&oauth_consumer_key=dpf43f3p2l4k3l03&oauth_nonce=kllo9940pd9333jh&oauth_signature_method=HMAC-SHA1&oauth_timestamp=1191242096&oauth_token=nnch734d00sl2jdk&oauth_version=1.0&size=original')

    def test_normalizes_query_spaces_in_first_name(self):
        query_dict = self._add_test_oauth_params_to_request_dict(self.spaces_params_first_name)

        normalized_qs = signature.normalize_query_parameters(query_dict)

        self.assertTrue(normalized_qs.index('%20'))

    def test_generates_base_string_spaces_in_first_name(self):
        query_dict = self._add_test_oauth_params_to_request_dict(self.spaces_params_first_name)
        query_dict['query'] = self.test_query_email

        normalized_qs = signature.normalize_query_parameters(query_dict)
        base_string = signature.concatenate_request_elements(self.method, self.url, normalized_qs)

        self.assertTrue(base_string.index('%2520'))

    def test_generates_base_string(self):
        query = signature.normalize_query_parameters(self.params)
        base_string = signature.concatenate_request_elements(self.method, self.url, query)
        self.assertEqual(base_string, 'GET&http%3A%2F%2Fphotos.example.net%2Fphotos&file%3Dvacation.jpg%26oauth_consumer_key%3Ddpf43f3p2l4k3l03%26oauth_nonce%3Dkllo9940pd9333jh%26oauth_signature_method%3DHMAC-SHA1%26oauth_timestamp%3D1191242096%26oauth_token%3Dnnch734d00sl2jdk%26oauth_version%3D1.0%26size%3Doriginal')

    def test_calculates_signature_value(self):
        result = signature.create(self.method, self.url, self.params, self.hmac_key)
        self.assertEqual(result, 'tR3+Ty81lMeYAr/Fid0kMTYa/WM=')


if __name__ == '__main__':
    unittest.main()



"""Package for Emailage."""

import sys

PYTHON_VERSION = 2, 7

if sys.version_info < PYTHON_VERSION:  # pragma: no cover (manual test)
    sys.exit("Python {}.{}+ is required.".format(*PYTHON_VERSION))


from emailage.client import TlsVersions
protocols = TlsVersions()





import os
import urllib
import unittest

from emailage.client import ApiDomains
from emailage.client import EmailageClient

use_urlparse = hasattr(urllib, 'quote')

if use_urlparse:
    import urlparse
    _parse_qs = urlparse.parse_qs
else:
    _parse_qs = urllib.parse.parse_qs


class IntegrationTest(unittest.TestCase):

    def setUp(self):
        self.api_secret = os.getenv('ENV_SECRET') or 'TEST_SECRET'
        self.api_token = os.getenv('ENV_TOKEN') or 'TEST_TOKEN'

        if self.api_secret == 'TEST_SECRET' and self.api_token == 'TEST_TOKEN':
            raise AssertionError("Ensure you have set valid login credentials in your environment")

        self.api_domain = os.getenv('ENV_DOMAIN') or ApiDomains.sandbox
        self.api_user_email = os.getenv('ENV_USER_EMAIL') or 'user.name@emailage.com'

        self.client_instance = EmailageClient(self.api_secret, self.api_token)
        self.client_instance.set_api_domain(self.api_domain)

        self.test_query_email = 'tumbled.email+inbox@gmail.com'
        self.test_query_ip = '13.25.10.245'

        self.no_spaces_params = {
            'first_name': 'Johann',
            'last_name': 'Vandergrift',
            'phone': '+14805559163'
        }

        self.spaces_params_first_name = {
            'first_name': 'Johann Paulus',
            'last_name': 'Vandergrift',
            'phone': '+14805559163'
        }

        self.spaces_params_last_name = {
            'first_name': 'Johann',
            'last_name': 'van der Grift',
            'phone': '+14805559163'
        }

        self.responseStatusSuccess = {
            'status': 'success',
            'errorCode': '0',
            'description': ''
        }

    def test_trivial_client_query(self):

        test_query_email = 'johann.vandergrift@emailage.com'

        params = {
            'user_email': self.api_user_email,
        }

        response = self.client_instance.query(test_query_email, **params)

        self.assertIsNotNone(response)
        self.assertDictEqual(response['responseStatus'], self.responseStatusSuccess)

        self.assertEqual(response['query']['email'], test_query_email)

    def test_trivial_email_ip_client_query(self):

        test_query_tuple = (self.test_query_email, self.test_query_ip)

        params = {
            'user_email': self.api_user_email
        }

        response = self.client_instance.query(test_query_tuple, **params)

        self.assertIsNotNone(response)
        self.assertDictEqual(response['responseStatus'], self.responseStatusSuccess)
        self.assertEqual(response['query']['email'], self.test_query_email + '+' + self.test_query_ip)

    def test_email_optional_params_no_spaces_client_query(self):

        self.no_spaces_params['user_email'] = self.api_user_email

        response = self.client_instance.query(self.test_query_email, **self.no_spaces_params)

        self.assertIsNotNone(response)
        self.assertDictEqual(response['responseStatus'], self.responseStatusSuccess)
        self.assertEqual(response['query']['email'], self.test_query_email)

    def test_email_ip_optional_params_no_spaces_client_query(self):

        test_query_tuple = (self.test_query_email, self.test_query_ip)

        self.no_spaces_params['user_email'] = self.api_user_email

        response = self.client_instance.query(test_query_tuple, **self.no_spaces_params)

        self.assertIsNotNone(response)
        self.assertDictEqual(response['responseStatus'], self.responseStatusSuccess)
        self.assertEqual(response['query']['email'], self.test_query_email + '+' + self.test_query_ip)

    def test_email_optional_params_first_name_space_client_query(self):

        self.spaces_params_first_name['user_email'] = self.api_user_email

        response = self.client_instance.query(self.test_query_email, **self.spaces_params_first_name)

        self.assertIsNotNone(response)
        self.assertDictEqual(response['responseStatus'], self.responseStatusSuccess)
        self.assertEqual(response['query']['email'], self.test_query_email)

    def test_email_ip_optional_params_first_name_space_client_query(self):

        test_query_tuple = (self.test_query_email, self.test_query_ip)

        self.spaces_params_first_name['user_email'] = self.api_user_email

        response = self.client_instance.query(test_query_tuple, **self.spaces_params_first_name)

        self.assertIsNotNone(response)
        self.assertDictEqual(response['responseStatus'], self.responseStatusSuccess)
        self.assertEqual(response['query']['email'], self.test_query_email + '+' + self.test_query_ip)

    def test_email_optional_params_last_name_space_client_query(self):

        self.spaces_params_last_name['user_email'] = self.api_user_email

        response = self.client_instance.query(self.test_query_email, **self.spaces_params_last_name)

        self.assertIsNotNone(response)
        self.assertDictEqual(response['responseStatus'], self.responseStatusSuccess)
        self.assertEqual(response['query']['email'], self.test_query_email)

    def test_email_ip_optional_params_last_name_space_client_query(self):

        test_query_tuple = (self.test_query_email, self.test_query_ip)

        self.spaces_params_last_name['user_email'] = self.api_user_email

        response = self.client_instance.query(test_query_tuple, **self.spaces_params_last_name)

        self.assertIsNotNone(response)
        self.assertDictEqual(response['responseStatus'], self.responseStatusSuccess)
        self.assertEqual(response['query']['email'], self.test_query_email + '+' + self.test_query_ip)


if __name__ == '__main__':
    unittest.main()



import json
import ssl
import sys
import urllib

from requests import Session
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.poolmanager import PoolManager

from emailage import signature, validation


use_urllib_quote = hasattr(urllib, 'quote')

if use_urllib_quote:
    def _url_encode_dict(qs_dict):
        return '&'.join(map(lambda pair: '='.join([urllib.quote(str(pair[0]), ''), urllib.quote(str(pair[1]), '')]),
                            sorted(qs_dict.items())))
elif sys.version_info >= (3, 5):
    # Python >= 3.5
    def _url_encode_dict(qs_dict):
        return urllib.parse.urlencode(qs_dict, quote_via=urllib.parse.quote)
else:
    # Python 3.x - 3.4
    def _url_encode_dict(qs_dict):
        return '&'.join(map(lambda pair: '='.join([urllib.parse.quote(str(pair[0]), ''),
                                                   urllib.parse.quote(str(pair[1]), '')]),
                            sorted(qs_dict.items())))


class TlsVersions:
    """An enumeration of the TLS versions supported by the Emailage API"""
    TLSv1_1 = ssl.PROTOCOL_TLSv1_1
    TLSv1_2 = ssl.PROTOCOL_TLSv1_2


class ApiDomains:
    """API URLs for the specified domains """
    sandbox = 'https://sandbox.emailage.com'
    production = 'https://api.emailage.com'


class EmailageClient:

    FRAUD_CODES = {
        1: 'Card Not Present Fraud',
        2: 'Customer Dispute (Chargeback)',
        3: 'First Party Fraud',
        4: 'First Payment Default',
        5: 'Identify Theft (Fraud Application)',
        6: 'Identify Theft (Account Take Over)',
        7: 'Suspected Fraud (Not Confirmed)',
        8: 'Synthetic ID',
        9: 'Other'
    }

    class Adapter(HTTPAdapter):
        def __init__(self, tls_version=TlsVersions.TLSv1_2):
            self._tls_version = tls_version
            super(EmailageClient.Adapter, self).__init__()

        def init_poolmanager(self, connections, maxsize, block=False):
            self.poolmanager = PoolManager(
                num_pools=connections,
                maxsize=maxsize,
                block=block,
                ssl_version=self._tls_version)

    def __init__(self, secret, token, sandbox=False, tls_version=TlsVersions.TLSv1_2):
        """ Creates an instance of the EmailageClient using the specified credentials and environment

            :param secret: Consumer secret, e.g. SID or API key.
            :param token: Consumer token.
            :param sandbox:
                (Optional) Whether to use a sandbox instead of a production server. Uses production by default
            :param tls_version: (Optional) Uses TLS version 1.2 by default (TlsVersions.TLSv1_2 | TlsVersions.TLSv1_1)

            :type secret: str
            :type token: str
            :type sandbox: bool
            :type tls_version: see :class:`TlsVersions`

            :Example:

            >>> from emailage.client import EmailageClient
            >>> from emailage import protocols
            >>> client = EmailageClient('consumer_secret', 'consumer_token', sandbox=True, tls_version=protocols.TLSv1_1)
            >>> fraud_report = client.query(('useremail@example.co.uk', '192.168.1.1'), urid='some_unique_identifier')

        """
        self.secret, self.token, self.sandbox = secret, token, sandbox
        self.hmac_key = token + '&'
        self.session = None
        self.domain = None
        self.set_api_domain((sandbox and ApiDomains.sandbox or ApiDomains.production), tls_version)

    def set_credentials(self, secret, token):
        """ Explicitly set the authentication credentials to be used when generating a request in the current session.
            Useful when you want to change credentials after initial creation of the client.

            :param secret: Consumer secret, e.g. SID or API key
            :param token: Consumer token
            :return: None

        """
        self.secret = secret
        self.token = token
        self.hmac_key = token + '&'

    def set_api_domain(self, domain, tls_version=TlsVersions.TLSv1_2):
        """ Explicitly set the API domain to use for a session of the client, typically used in testing scenarios

            :param domain: API domain to use for the session
            :param tls_version: (Optional) Uses TLS version 1.2 by default (TlsVersions.TLSv1_2 | TlsVersions.TLSv1_1)
            :return: None

            :type domain: str see :class: `ApiDomains`
            :type tls_version: see :class: `TlsVersions`

            :Example:

            >>> from emailage.client import EmailageClient
            >>> from emailage.client import ApiDomains
            >>> client = EmailageClient('consumer_secret', 'consumer_token')
            >>> client.set_api_domain(ApiDomains.sandbox)
            >>> client.domain
            'https://sandbox.emailage.com'

            :Example:

            >>> from emailage.client import EmailageClient
            >>> client = EmailageClient('consumer_secret', 'consumer_token')
            >>> client.set_api_domain('https://testing.emailage.com')
            >>> client.domain
            'https://testing.emailage.com'
        """
        self.session = Session()
        self.session.headers.update({
            'Content-Type': 'application/json'
        })
        self.domain = domain
        self.session.mount(self.domain, EmailageClient.Adapter(tls_version))
    
    def request(self, endpoint, **params):
        """ Base method to generate requests for the Emailage validator and flagging APIs

            :param endpoint: API endpoint to send the request ( '' | '/flag' )
            :param params: keyword-argument list of parameters to send with the request
            :return: JSON dict of the response generated by the API

            :type endpoint: str
            :type params: kwargs

            :Example:

            >>> from emailage.client import EmailageClient
            >>> client = EmailageClient('consumer_secret', 'consumer_token')
            >>> response = client.request('/flag', email='user20180830001@domain20180830001.com', flag='good')
            >>> response['query']['email']
            u'user20180830001%40domain20180830001.com'
        """
        url = self.domain + '/emailagevalidator' + endpoint + '/'

        params = dict(
            format='json',
            **params
        )
        params = signature.add_oauth_entries_to_fields_dict(self.secret, params)
        params['oauth_signature'] = signature.create('GET', url, params, self.hmac_key)
        params_qs = _url_encode_dict(params)

        res = self.session.get(url, params=params_qs)
      
        # Explicit encoding is necessary because the API returns a Byte Order Mark at the beginning of the contents
        json_data = res.content.decode(encoding='utf_8_sig')
        return json.loads(json_data)

    def query(self, query, **params):
        """ Base query method providing support for email, IP address, and optional additional parameters
        
            :param query: RFC2822-compliant Email, RFC791-compliant IP, or both
            :param params: keyword-argument form for parameters such as urid, first_name, last_name, etc.
            :return: JSON dict of the response generated by the API

            :type query: str | (str, str)
            :type params: kwargs

            :Example:

            >>> from emailage.client import EmailageClient
            >>> client = EmailageClient('consumer_secret', 'consumer_token')
            >>> response_json = client.query('test@example.com')
            >>> # Email address only
            >>> response_json = client.query('test@example.com')
            >>> # IP Address only
            >>> response_json = client.query('209.85.220.41')
            >>> # For a combination. Please note the order
            >>> response_json = client.query(('test@example.com', '209.85.220.41'))
            >>> # Pass a User Defined Record ID (URID) as an optional parameter
            >>> response_json = client.query('test@example.com', urid='My record ID for test@example.com')
        """
        if type(query) is tuple:
            validation.assert_email(query[0])
            validation.assert_ip(query[1])
            query = '+'.join(query)
        params['query'] = query
        return self.request('', **params)
    
    def query_email(self, email, **params):
        """Query a risk score information for the provided email address.

            :param email: RFC2822-compliant Email
            :param params: (Optional) keyword-argument form for parameters such as urid, first_name, last_name, etc.

            :return: JSON dict of the response generated by the API

            :type email: str
            :type params: kwargs

            :Example:

            >>> from emailage.client import EmailageClient
            >>> client = EmailageClient('My account SID', 'My auth token', sandbox=True)
            >>> response_json = client.query_email('test@example.com')
        """
        validation.assert_email(email)
        return self.query(email, **params)
    
    def query_ip_address(self, ip, **params):
        """Query a risk score information for the provided IP address.

            :param ip: RFC791-compliant IP
            :param params: (Optional) keyword-argument form for parameters such as urid, first_name, last_name, etc.
            :return: JSON dict of the response generated by the API

            :type ip: str
            :type params: kwargs

            :Example:

            >>> from emailage.client import EmailageClient
            >>> client = EmailageClient('My account SID', 'My auth token', sandbox=True)
            >>> response_json = client.query_ip_address('209.85.220.41')
        """
        validation.assert_ip(ip)
        return self.query(ip, **params)
    
    def query_email_and_ip_address(self, email, ip, **params):
        """Query a risk score information for the provided combination of an Email and IP address

            :param email: RFC2822-compliant Email
            :param ip: RFC791-compliant IP
            :param params: (Optional) keyword-argument form for parameters such as urid, first_name, last_name, etc.
            :return: JSON dict of the response generated by the API

            :type email: str
            :type ip: str
            :type params: kwargs

            :Example:

            >>> from emailage.client import EmailageClient
            >>> client = EmailageClient('My account SID', 'My auth token', sandbox=True)
            >>> response_json = client.query_email_and_ip_address('test@example.com', '209.85.220.41')
            >>> response_json = client.query_email_and_ip_address('test@example.com', '209.85.220.41', urid='My record ID for test@example.com and 209.85.220.41')

        """
        validation.assert_email(email)
        validation.assert_ip(ip)
        return self.query((email, ip), **params)

    def flag(self, flag, query, fraud_code=None):
        """ Base method used to flag an email address as fraud, good, or neutral

            :param flag: type of flag you wish to associate with the identifier ( 'fraud' | 'good' | 'neutral' )
            :param query: Email to be flagged
            :param fraud_code:
                (Optional) Required if flag is 'fraud', one of the IDs in `emailage.client.EmailageClient.FRAUD_CODES`

            :return: JSON dict of the confirmation response generated by the API

            :type flag: str
            :type query: str
            :type fraud_code: int

            :Example:

            >>> from emailage.client import EmailageClient
            >>> client = EmailageClient('My account SID', 'My auth token', sandbox=True)
            >>> response_json = client.flag('good', 'test@example.com')
            >>> response_json = client.flag('fraud', 'test@example.com', fraud_code=6)
            >>> response_json = client.flag('neutral', 'test@example.com')

        """
        flags = ['fraud', 'neutral', 'good']
        if flag not in flags:
            raise ValueError(validation.Messages.FLAG_NOT_ALLOWED_FORMAT.format(', '.join(flags), flag))

        validation.assert_email(query)
        
        params = dict(flag=flag, query=query)

        if flag == 'fraud':
            codes = self.FRAUD_CODES
            if type(fraud_code) is not int:
                raise ValueError(
                    validation.Messages.FRAUD_CODE_RANGE_FORMAT.format(
                        len(codes), ', '.join(codes.values()), fraud_code)
                )
            if fraud_code not in range(1, len(codes) + 1):
                fraud_code = 9
            params['fraudcodeID'] = fraud_code

        return self.request('/flag', **params)
    
    def flag_as_fraud(self, query, fraud_code):
        """Mark an email address as fraud.

            :param query: Email to be flagged
            :param fraud_code: Reason for the email to be marked as fraud; must be one of the IDs in `emailage.client.EmailageClient.FRAUD_CODES`
            :return: JSON dict of the confirmation response generated by the API

            :type query: str
            :type fraud_code: int

            :Example:

            >>> from emailage.client import EmailageClient
            >>> client = EmailageClient('My account SID', 'My auth token', sandbox=True)
            >>> response_json = client.flag_as_fraud('test@example.com', 8)

        """
        return self.flag('fraud', query, fraud_code)
    
    def flag_as_good(self, query):
        """Mark an email address as good.

            :param query: Email to be flagged
            :return: JSON dict of the confirmation response generated by the API

            :type query: str

            :Example:

            >>> from emailage.client import EmailageClient
            >>> client = EmailageClient('My account SID', 'My auth token', sandbox=True)
            >>> response_json = client.flag_as_good('test@example.com')

        """
        return self.flag('good', query)
    
    def remove_flag(self, query):
        """Unflag an email address that was marked as good or fraud previously.

            :param query: Email to be flagged
            :return: JSON dict of the confirmation response generated by the API

            :type query: str

            :Example:

            >>> from emailage.client import EmailageClient
            >>> client = EmailageClient('My account SID', 'My auth token', sandbox=True)
            >>> response_json = client.remove_flag('test@example.com')

        """
        return self.flag('neutral', query)






#!/usr/bin/env python

"""Setup script for Emailage."""

import setuptools

try:
    README = open("README.md").read()
    CHANGES = open("CHANGES.md").read()
except IOError:
    DESCRIPTION = "The Emailage (TM) API was built to help companies integrate with our highly efficient fraud risk and scoring system. By calling our API endpoints and simply passing us an email and/or IP Address, companies will be provided with real-time risk scoring assessments based around machine learning and proprietary algorithms that evolve with new fraud trends."
else:
    DESCRIPTION = README + CHANGES

PROJECT = 'emailage-official'
VERSION = '1.1.7'

setuptools.setup(
    name=PROJECT,
    version=VERSION,

    description="Official Emailage API client written in Python",
    url='https://www.emailage.com/',
    author='Emailage Dev Team',
    author_email='dev@emailage.com',

    packages=setuptools.find_packages(),

    entry_points={'console_scripts': []},

    long_description=DESCRIPTION,
    long_description_content_type='text/markdown',
    license='MIT',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7'
    ],

    install_requires=open("requirements.txt").readlines(),
)



import unittest
import urllib
from mock import Mock
from emailage import protocols
from emailage.client import EmailageClient


use_urlparse = hasattr(urllib, 'quote')

if use_urlparse:
    import urlparse
    _parse_qs = urlparse.parse_qs
else:
    _parse_qs = urllib.parse.parse_qs


class ClientTest(unittest.TestCase):

    _tls_version = None

    def setUp(self):
        response = Mock()
        response.content = b'\xEF\xBB\xBF{"success":[true]}'
        response.text = str(response.content)
        
        self.email = 'test+emailage@example.com'
        self.ip = '1.234.56.7'

        self.no_spaces_params = {
            'first_name': 'Johann',
            'last_name': 'Vandergrift',
            'phone': '+14805559163'
        }

        self.spaces_params_first_name = {
            'first_name': 'Johann Paulus',
            'last_name': 'Vandergrift',
            'phone': '+14805559163'
        }

        self.spaces_params_last_name = {
            'first_name': 'Johann',
            'last_name': 'van der Grift',
            'phone': '+14805559163'
        }

        if self._tls_version is not None:
            self.subj = EmailageClient('secret', 'token', sandbox=True, tls_version=self._tls_version)
        else:
            self.subj = EmailageClient('secret', 'token', sandbox=True)

        self.g = self.subj.session.get = Mock(return_value=response)

    def test_is_initialized_with_properties(self):
        self.assertTrue(self.subj.sandbox)

    def test_generates_appropriate_key(self):
        self.assertEqual(self.subj.hmac_key, 'token&')


class ClientRequestTest(ClientTest):
    
    def _request(self):
        return self.subj.request('/endpoint', query='something')
    
    def test_request__call(self):
        """Targets requests to the correct endpoint with correct request params"""
        self._request()
        call_args = self.g.call_args_list[0]
        url = call_args[0][0]
        params = _parse_qs(call_args[1]['params'])

        self.assertEqual(url, 'https://sandbox.emailage.com/emailagevalidator/endpoint/')
        self.assertEqual(params['query'][0], 'something')
        self.assertEqual(params['oauth_consumer_key'][0], 'secret')
    
    def test_request__returns(self):
        """Parses response body as JSON"""
        self.assertEqual(self._request(), {'success': [True]})
        
        
class ClientQueryTest(ClientTest):
    
    def test_query(self):
        """Concatenates arguments and passes extra params to request"""
        self.r = self.subj.request = Mock()
        self.subj.query((self.email, self.ip), urid=1234567890)
        
        self.r.assert_called_once_with('', urid=1234567890, query='test+emailage@example.com+1.234.56.7')
        
        
class ClientFlagTest(ClientTest):
    
    def setUp(self):
        super(ClientFlagTest, self).setUp()
        self.r = self.subj.request = Mock()

    def test_flag_as_fraud__call(self):
        """Flags a supplied address as fraud with an appropriate fraud code"""
        self.subj.flag_as_fraud(self.email, 3)
        
        self.r.assert_called_once_with('/flag', flag='fraud', query='test+emailage@example.com', fraudcodeID=3)

    def test_flag_as_fraud__call2(self):
        """Flags a supplied address as fraud with fraud code = 9 if the code is an integer but out of range"""
        self.subj.flag_as_fraud(self.email, 42)
        
        self.r.assert_called_once_with('/flag', flag='fraud', query='test+emailage@example.com', fraudcodeID=9)
            
    def test_flag_as_fraud__exceptions(self):
        """Raises an error when none, unknown or string fraud code is supplied"""
        self.assertRaises(TypeError, self.subj.flag_as_fraud, self.email)
        self.assertRaises(ValueError, self.subj.flag_as_fraud, self.email, 'Blah blah')
        self.assertRaises(ValueError, self.subj.flag_as_fraud, self.email, 'First Party Fraud')
    
    def test_flag_as_good(self):
        """Flags a supplied address as good"""
        self.subj.flag_as_good(self.email)
        
        self.r.assert_called_once_with('/flag', flag='good', query=self.email)

    def test_remove_flag(self):
        """Unflags a supplied address"""
        self.subj.remove_flag(self.email)
        
        self.r.assert_called_once_with('/flag', flag='neutral', query=self.email)


class ClientTls11Test(ClientQueryTest, ClientRequestTest):

    @classmethod
    def setUpClass(cls):
        cls._tls_version = protocols.TLSv1_1


class ClientTls12Test(ClientQueryTest, ClientRequestTest):

    @classmethod
    def setUpClass(cls):
        cls._tls_version = protocols.TLSv1_2


class ClientTls11FlagTest(ClientFlagTest):

    @classmethod
    def setUpClass(cls):
        cls._tls_version = protocols.TLSv1_1


class ClientTls12FlagTest(ClientFlagTest):

    @classmethod
    def setUpClass(cls):
        cls._tls_version = protocols.TLSv1_2


if __name__ == '__main__':
    unittest.main()

